pub struct LSMTree<K, V> {
    memlog: WritableMemLog<K, OptDatum<V>>,
    sstables: Vec<SSTable<K, OptDatum<V>>>,
    sstables_dir: AntiCollisionParentDir,
}
An LSMTree is an abstraction of a sorted dictionary.

API:

The exposed operations are: put one, get one, get range.

Values are immutable. They cannot be modified in-place, and must be replaced.

Internals:

One [WritableMemLog] holds the most recently inserted {key: value} in a sorted in-memory table.

The [WritableMemLog] is occasionally flushed into an [SSTable].

Multiple [SSTable]s are occasionally compacted into one [SSTable].

Querying:

A put operation accesses the Memtable of the [WritableMemLog] only.

A get operation generally accesses the [WritableMemLog] and all [SSTable]s.

When the same key exists in multiple internal tables, only the result from the newest table is retrieved.

Fields

memlog: WritableMemLog<K, OptDatum<V>>sstables: Vec<SSTable<K, OptDatum<V>>>

From older to newer.

sstables_dir: AntiCollisionParentDir

Implementations

source

impl<K, V> LSMTree<K, V>
where K: Serializable + Ord + Clone, OptDatum<V>: Serializable,

source

pub fn maybe_run_gc(&mut self) -> Result<()>

source

fn flush_memtable(&mut self) -> Result<()>

source

fn compact_sstables(&mut self) -> Result<()>

For now, always compact all SSTables into one SSTable.

source

impl<K, V> LSMTree<K, V>
where K: Serializable + Ord + Clone, OptDatum<V>: Serializable,

source

pub fn put(&mut self, k: K, v: Option<V>) -> Result<()>

source

pub fn get_one<'a, Q>(&'a self, k: &'a Q) -> Option<Entry<'a, K, V>>
where K: Borrow<Q> + PartialOrd<Q>, Q: Ord,

source

pub fn get_range<'a, Q>( &'a self, k_lo: Option<&'a Q>, k_hi: Option<&'a Q> ) -> impl 'a + Iterator<Item = Entry<'a, K, V>>
where K: PartialOrd<Q>,

source

pub fn get_whole_range<'a>( &'a self ) -> impl 'a + Iterator<Item = Entry<'a, K, V>>

source

impl<K, V> LSMTree<K, V>
where K: Serializable + Ord + Clone, OptDatum<V>: Serializable,

source

pub fn load_or_new<P: AsRef<Path>>(lsm_dir_path: P) -> Result<Self>